@@ -37,4 +37,4 @@ est <- fun()
 rm(x2)
 summary(est, robust=TRUE)
 for(lh in est$stage1$lhs) print(summary(est$stage1, lhs=lh))
-print(condfstat(est,NULL))
+print(condfstat(est,NULL,quantiles=c(0.1,0.5,0.9)))
